@@ -41,9 +41,9 @@ class ValueNode(Node):
         self.value = value
     
     def generate(self, i) -> str:
-        return i * " " +  "ValueNode: " + str(self.value) + "\n"
+        return i * " " + "ValueNode: " + str(self.value) + "\n"
     
-    def rrename(self, vars_: dict) -> str:
+    def rrename(self, vars_: dict) -> Node:
         if "." in self.value:
             spl = self.value.split(".")
             if len(spl) == 2:
@@ -51,6 +51,32 @@ class ValueNode(Node):
         
         return ValueNode(self.pos, vars_.get(self.value, self.value))
 
+class IndexNode(Node):
+    def __init__(self, pos: Position, var: str, index: Node):
+        self.pos = pos
+        self.var = var
+        self.index = index
+    
+    def generate(self, i) -> str:
+        return i * " " + "IndexNode: " + str(self.var) + ", " + self.index.generate(i + 1) + "\n"
+    
+    def rrename(self, vars_: dict) -> Node:
+        return IndexNode(self.pos, vars_.get(self.var, self.var), self.index.rrename(vars_))
+
+class IndexAssignNode(Node):
+    def __init__(self, pos: Position, var: str, index: Node, atype: str, val: Node):
+        self.pos = pos
+        self.var = var
+        self.index = index
+        self.atype = atype
+        self.val = val
+    
+    def generate(self, i) -> str:
+        return i * " " + "IndexNode: " + str(self.var) + ", " + self.index.generate(i + 1) + ", " + self.atype + ", " + self.val.generate(i + 1) + "\n"
+    
+    def rrename(self, vars_: dict) -> Node:
+        return IndexNode(self.pos, vars_.get(self.var, self.var), self.index.rrename(vars_), self.atype, self.val.rrename(vars_))
+
 class AtomNode(Node):
     def __init__(self, pos: Position, value):
         self.pos = pos
@@ -168,6 +194,19 @@ class CallNode(Node):
     def rrename(self, vars_: dict) -> Node:
         return CallNode(self.pos, self.function if type(self.function) == str else self.function.rrename(vars_), self.is_call, [n.rrename(vars_) for n in self.params])
 
+class SubCallNode(Node):
+    def __init__(self, pos: Position, function: str, params: list = []):
+        self.pos = pos
+        self.function = function
+        self.params = params
+    
+    def generate(self, i) -> str:
+        return i * " " + f"SubCallNode: {self.function}({','.join(self.params)})\n"
+    
+    def rrename(self, vars_: dict) -> Node:
+        return SubCallNode(self.pos, self.function, [n.rrename(vars_) for n in self.params])
+
+
 class KeywordNode(Node):
     def __init__(self):
         pass
@@ -359,6 +398,35 @@ class Parser:
                 
                 return AssignmentNode(id_.pos(), id_.value, n.value, self.parse_ExpressionNode())
             elif n.type == TokenType.DOT:
+                if id_.value in functions.native_sub:
+                    n_ = self.next_token(TokenType.ID)
+                    
+                    if not n_.value in functions.native_sub[id_.value]:
+                        parse_error(n_, "Invalid subcommand")
+
+                    self.next_token(TokenType.LPAREN)
+
+                    e = False
+                    p = []
+                    while True:
+                        t = self.next_token()
+                        if t.type == TokenType.RPAREN:
+                            break
+                        elif t.type == TokenType.COMMA:
+                            if e:
+                                p.append(self.parse_ExpressionNode())
+                            else:
+                                parse_error(t, "Unexpected token")
+                        else:
+                            if not e:
+                                self.pos -= 1
+                                p.append(self.parse_ExpressionNode())
+                                e = True
+                            else:
+                                parse_error(t, "Unexpected token")
+                    
+                    return SubCallNode(id_.pos(), f"{id_.value}.{n_.value}", p)
+
                 if id_.value in functions.special:
                     parse_error(n, "Cannot use property of a keyword")
 
@@ -369,6 +437,18 @@ class Parser:
                 node.left = id_.value + "." + node.left
 
                 return node
+            elif n.type == TokenType.LBRACK:
+                if id_.value in functions.special:
+                    parse_error(n, "Cannot use keyword as a memory cell")
+                
+                node = self.parse_ExpressionNode()
+
+                self.next_token(TokenType.RBRACK)
+                a = self.next_token(TokenType.SET)
+
+                val = self.parse_ExpressionNode()
+
+                return IndexAssignNode(id_.pos(), id_.value, node, a.value, val)
             elif n.type == TokenType.LPAREN:
                 if id_.value in functions.keywords:
                     if not can_be_special:
@@ -527,6 +607,10 @@ class Parser:
                 if n_.type == TokenType.DOT:
                     n_ = self.next_token(TokenType.ID)
                     return AtomNode(n.pos(), ValueNode(n.pos(), f"{n.value}.{n_.value}"))
+                elif n_.type == TokenType.LBRACK:
+                    n_ = self.parse_ExpressionNode()
+                    self.next_token(TokenType.RBRACK)
+                    return AtomNode(n.pos(), IndexNode(n.pos(), n.value, n_))
                 else:
                     self.pos -= 1
             
